#!/usr/bin/env python
# coding: utf-8

import sys, netrc, shlex, ssl, threading
import irc.bot, irc.strings
import paramiko


class SydBot(irc.bot.SingleServerIRCBot):
    def __init__(self, channels, nickname, server, port=6697):
        credentials = netrc.netrc().authenticators(server)
        password = credentials[2] if credentials else None

        ssl_factory = irc.connection.Factory(wrapper=ssl.wrap_socket)
        irc.bot.SingleServerIRCBot.__init__(
            self,
            [(server, port, password)],
            nickname,
            nickname,
            connect_factory=ssl_factory,
        )
        self.channel_list = channels

    def on_nicknameinuse(self, c, e):
        c.nick(c.get_nickname() + "_")

    def on_welcome(self, c, e):
        for channel in self.channel_list:
            c.join(channel)
            print(f"Joining {channel}", file=sys.stderr)

    def on_privmsg(self, c, e):
        print(f"Received command: {e.arguments[0]}", file=sys.stderr)
        self.do_command(e, e.arguments[0], e.source.nick)

    def on_pubmsg(self, c, e):
        cmd = None
        if e.arguments[0].startswith(";"):
            cmd = e.arguments[0][1:]
            sh = "/usr/bin/rc"
        elif e.arguments[0].startswith("$"):
            cmd = e.arguments[0][1:]
            sh = "/bin/bash"

        if cmd is not None:
            print(
                f"Received command: {cmd} in {e.target} using shell {sh}",
                file=sys.stderr,
            )
            self.do_command(e, e.arguments[0][1:], e.target, sh=sh)

    def do_command(self, e, cmd, target, sh="/bin/bash"):
        c = self.connection

        def execute_ssh_command():
            ssh = paramiko.SSHClient()
            ssh.set_missing_host_key_policy(paramiko.AutoAddPolicy())
            ssh.connect("syd.chesswob.org", username="syd", password="syd")

            safe_cmd = shlex.quote(cmd)
            ssh_stdin, ssh_stdout, ssh_stderr = ssh.exec_command(
                f"timeout -v --foreground -sKILL 7s {sh} -c {safe_cmd}"
            )
            print("Executed command, waiting for output...", file=sys.stderr)
            try:
                output = ssh_stdout.read().decode(
                    "utf-8", "ignore"
                ) + ssh_stderr.read().decode("utf-8", "ignore")
                output = " ".join(
                    output.split()
                )  # Replacing newlines with spaces and removing extra spaces
                if len(output) > 256:
                    # Send first 256 chars
                    output = output[:256] + "…"
                if not output:
                    output = "<no output>"
                print(f"Sending output '{output}' to {target}", file=sys.stderr)
                c.privmsg(target, output)
            except paramiko.SSHException:
                c.privmsg(target, "Command timed out after 7 seconds.")

            ssh.close()

        # Run SSH command in a separate thread to avoid blocking the bot
        thread = threading.Thread(target=execute_ssh_command)
        thread.start()


def main():
    if len(sys.argv) != 4:
        print("Usage: sydbot <server[:port]> <channel,...> <nickname>")
        sys.exit(1)

    s = sys.argv[1].split(":", 1)
    server = s[0]
    if len(s) == 2:
        try:
            port = int(s[1])
        except ValueError:
            print("Error: Erroneous port.")
            sys.exit(1)
    else:
        port = 6697
    channels = sys.argv[2].split(",")
    nickname = sys.argv[3]

    bot = SydBot(channels, nickname, server, port)
    bot.start()


if __name__ == "__main__":
    main()
